@@ -1,23 +1,23 @@
 methods::setGeneric("embedded", function(data, ...) standardGeneric("embedded"))
 
-.embedded_sf_method = \(data,target,E = 3,nb = NULL,include.self = FALSE){
+.embedded_sf_method = \(data,target,E = 3,tau = 1,nb = NULL){
   vec = .uni_lattice(data,target)
-  if (is.null(nb)) nb = sdsfun::spdep_nb(data)
-  return(RcppGenLatticeEmbeddings(vec,nb,E,include.self))
+  if (is.null(nb)) nb = .internal_lattice_nb(data)
+  return(RcppGenLatticeEmbeddings(vec,nb,E,tau))
 }
 
-.embedded_spatraster_method = \(data,target,E = 3,include.self = FALSE){
+.embedded_spatraster_method = \(data,target,E = 3,tau = 1){
   mat = .uni_grid(data,target)
-  return(RcppGenGridEmbeddings(mat,E,include.self))
+  return(RcppGenGridEmbeddings(mat,E,tau))
 }
 
 #' generate embeddings
 #'
 #' @param data The observation data.
 #' @param target Name of target variable.
-#' @param E (optional) The dimensions of the embedding.
+#' @param E (optional) Dimensions of the embedding.
+#' @param tau (optional) Step of spatial lags.
 #' @param nb (optional) The neighbours list.
-#' @param include.self (optional) Whether to include the current state when constructing the embedding vector.
 #'
 #' @return A matrix
 #' @export
@@ -27,8 +27,7 @@ methods::setGeneric("embedded", function(data, ...) standardGeneric("embedded"))
 #' @aliases embedded,sf-method
 #'
 #' @examples
-#' columbus = sf::read_sf(system.file("shapes/columbus.gpkg", package="spData")[1],
-#'                        quiet=TRUE)
+#' columbus = sf::read_sf(system.file("shapes/columbus.gpkg", package="spData"))
 #' embedded(columbus,target = "CRIME", E = 3)
 #'
 methods::setMethod("embedded", "sf", .embedded_sf_method)
