@@ -119,8 +119,8 @@ double CppRMSE(const std::vector<double>& vec1,
 }
 
 // Function to calculate the absolute difference between two vectors
-std::vector<double> CppAbs(const std::vector<double>& vec1,
-                           const std::vector<double>& vec2) {
+std::vector<double> CppAbsDiff(const std::vector<double>& vec1,
+                               const std::vector<double>& vec2) {
   if (vec1.size() != vec2.size()) {
     throw std::invalid_argument("Vectors must have the same size");
   }
@@ -187,6 +187,45 @@ double CppCovariance(const std::vector<double>& vec1,
   return count > 1 ? cov / (count - 1) : std::numeric_limits<double>::quiet_NaN();
 }
 
+// Function to compute distance between two vectors:
+double CppDistance(const std::vector<double>& vec1,
+                   const std::vector<double>& vec2,
+                   bool L1norm = false,
+                   bool NA_rm = false){
+  // Handle NA values
+  std::vector<double> clean_v1, clean_v2;
+  for (size_t i = 0; i < vec1.size(); ++i) {
+    bool is_na = isNA(vec1[i]) || isNA(vec2[i]);
+    if (is_na) {
+      if (!NA_rm) {
+        return std::numeric_limits<double>::quiet_NaN(); // Return NaN if NA_rm is false
+      }
+    } else {
+      clean_v1.push_back(vec1[i]);
+      clean_v2.push_back(vec2[i]);
+    }
+  }
+
+  // If no valid data, return NaN
+  if (clean_v1.empty()) {
+    return std::numeric_limits<double>::quiet_NaN();
+  }
+
+  double dist_res = 0.0;
+  if (L1norm) {
+    for (std::size_t i = 0; i < clean_v1.size(); ++i) {
+      dist_res += std::abs(clean_v1[i] - clean_v2[i]);
+    }
+  } else {
+    for (std::size_t i = 0; i < clean_v1.size(); ++i) {
+      dist_res += (clean_v1[i] - clean_v2[i]) * (clean_v1[i] - clean_v2[i]);
+    }
+    dist_res = std::sqrt(dist_res);
+  }
+
+  return dist_res;
+}
+
 // Function to compute Pearson correlation using Armadillo
 double PearsonCor(const std::vector<double>& y,
                   const std::vector<double>& y_hat,
@@ -233,9 +272,10 @@ double PearsonCor(const std::vector<double>& y,
 // double PearsonCor(const std::vector<double>& y,
 //                   const std::vector<double>& y_hat,
 //                   bool NA_rm = false) {
-//   if (y.size() != y_hat.size()) {
-//     throw std::invalid_argument("Vectors must have the same size");
-//   }
+//   // // Check input sizes
+//   // if (y.size() != y_hat.size()) {
+//   //   throw std::invalid_argument("Input vectors must have the same size.");
+//   // }
 //
 //   // Handle NA values
 //   std::vector<double> clean_y, clean_y_hat;
@@ -416,6 +456,43 @@ std::vector<double> CppCorConfidence(double r, int n, int k = 0,
   return {r_upper, r_lower};
 }
 
+// Function to find k-nearest neighbors of a given index in the embedding space
+std::vector<std::size_t> CppKNNIndice(
+    const std::vector<std::vector<double>>& embedding_space,
+    std::size_t target_idx,
+    std::size_t k)
+{
+  std::size_t n = embedding_space.size();
+  std::vector<std::pair<double, std::size_t>> distances;
+
+  for (std::size_t i = 0; i < n; ++i) {
+    if (i == target_idx) continue;
+
+    // Check if the entire embedding_space[i] is NaN
+    if (std::all_of(embedding_space[i].begin(), embedding_space[i].end(),
+                    [](double v) { return std::isnan(v); })) {
+      continue;
+    }
+
+    double dist = CppDistance(embedding_space[target_idx], embedding_space[i], false, true);
+
+    // Skip NaN distances
+    if (!std::isnan(dist)) {
+      distances.emplace_back(dist, i);
+    }
+  }
+
+  // Partial sort to get k-nearest neighbors, excluding NaN distances
+  std::partial_sort(distances.begin(), distances.begin() + std::min(k, distances.size()), distances.end());
+
+  std::vector<std::size_t> neighbors;
+  for (std::size_t i = 0; i < k && i < distances.size(); ++i) {
+    neighbors.push_back(distances[i].second);
+  }
+
+  return neighbors;
+}
+
 // Function to compute SVD similar to R's svd()
 // Input:
 //   - X: A matrix represented as std::vector<std::vector<double>>
