@@ -15,17 +15,36 @@
 // [[Rcpp::plugins(cpp11)]]
 // [[Rcpp::depends(RcppThread)]]
 
+/**
+ * @brief Computes the partial correlation between the target variable and its simplex projection,
+ *        incorporating control variables using a lattice-based embedding approach.
+ *
+ * @param vectors: Reconstructed state-space, where each row represents a separate state vector.
+ * @param target: Spatial cross-section series to be used as the target, aligned with 'vectors'.
+ * @param controls: Cross-sectional data of control variables, stored row-wise.
+ * @param nb_vec: Neighbor indices for each spatial unit.
+ * @param lib_indices: Boolean vector indicating which states to include when searching for neighbors.
+ * @param pred_indices: Boolean vector indicating which states to use for predictions.
+ * @param conEs: Vector specifying the number of dimensions for attractor reconstruction with control variables.
+ * @param taus: Vector specifying the spatial lag step for constructing lagged state-space vectors with control variables.
+ * @param num_neighbors: Number of neighbors to use for simplex projection.
+ * @param cumulate: Flag indicating whether to cumulatively incorporate control variables.
+ *
+ * @return A std::vector<double> containing:
+ *         - rho[0]: Pearson correlation between the target and its simplex projection.
+ *         - rho[1]: Partial correlation controlling for the influence of the control variables.
+ */
 std::vector<double> PartialSimplex4Lattice(
-    const std::vector<std::vector<double>>& vectors,  // Reconstructed state-space (each row is a separate vector/state)
-    const std::vector<double>& target,                // Spatial cross-section series to be used as the target (should line up with vectors)
-    const std::vector<std::vector<double>>& controls, // Cross-sectional data of control variables (**stored by row**)
-    const std::vector<std::vector<int>>& nb_vec,      // Neighbor indices vector of the spatial units
-    const std::vector<bool>& lib_indices,             // Vector of T/F values (which states to include when searching for neighbors)
-    const std::vector<bool>& pred_indices,            // Vector of T/F values (which states to predict from)
-    const std::vector<int>& conEs,                    // Number of dimensions for the attractor reconstruction with control variables
-    int num_neighbors,                                // Number of neighbors to use for simplex projection
-    bool cumulate,                                    // Whether to cumulate the partial correlations
-    bool includeself                                  // Whether to include the current state when constructing the embedding vector
+    const std::vector<std::vector<double>>& vectors,
+    const std::vector<double>& target,
+    const std::vector<std::vector<double>>& controls,
+    const std::vector<std::vector<int>>& nb_vec,
+    const std::vector<bool>& lib_indices,
+    const std::vector<bool>& pred_indices,
+    const std::vector<int>& conEs,
+    const std::vector<int>& taus,
+    int num_neighbors,
+    bool cumulate
 ){
   int n_controls = controls.size();
   std::vector<double> rho(2);
@@ -40,7 +59,7 @@ std::vector<double> PartialSimplex4Lattice(
       } else {
         temp_pred = SimplexProjectionPrediction(temp_embedding, controls[i], lib_indices, pred_indices, num_neighbors);
       }
-      temp_embedding = GenLatticeEmbeddings(temp_pred,nb_vec,conEs[i],includeself);
+      temp_embedding = GenLatticeEmbeddings(temp_pred,nb_vec,conEs[i],taus[i]);
     }
 
     std::vector<double> con_pred = SimplexProjectionPrediction(temp_embedding, target, lib_indices, pred_indices, num_neighbors);
@@ -55,7 +74,7 @@ std::vector<double> PartialSimplex4Lattice(
 
     for (int i = 0; i < n_controls; ++i) {
       temp_pred = SimplexProjectionPrediction(vectors, controls[i], lib_indices, pred_indices, num_neighbors);
-      temp_embedding = GenLatticeEmbeddings(temp_pred,nb_vec,conEs[i],includeself);
+      temp_embedding = GenLatticeEmbeddings(temp_pred,nb_vec,conEs[i],taus[i]);
       temp_pred = SimplexProjectionPrediction(temp_embedding, target, lib_indices, pred_indices, num_neighbors);
       con_pred[i] = temp_pred;
     }
@@ -67,18 +86,41 @@ std::vector<double> PartialSimplex4Lattice(
   return rho;
 }
 
+/**
+ * @brief Computes the partial correlation between a spatial cross-section series and its prediction
+ *        using the S-Map method, incorporating control variables.
+ *
+ * This function performs state-space reconstruction and S-Map prediction while accounting for
+ * control variables in a lattice-based spatial setting. The process can be either cumulative or
+ * independent in terms of incorporating control variables.
+ *
+ * @param vectors: Reconstructed state-space where each row represents a separate vector/state.
+ * @param target: Spatial cross-section series used as the prediction target.
+ * @param controls: Cross-sectional data of control variables, stored row-wise.
+ * @param nb_vec: Neighbor indices vector specifying spatial unit neighbors.
+ * @param lib_indices: Boolean vector indicating which states to include when searching for neighbors.
+ * @param pred_indices: Boolean vector indicating which states to predict from.
+ * @param conEs: Vector specifying the number of dimensions for attractor reconstruction with control variables.
+ * @param taus: Vector specifying the spatial lag step for constructing lagged state-space vectors with control variables.
+ * @param num_neighbors: Number of neighbors to use for S-Map prediction.
+ * @param theta: Weighting parameter for distances in S-Map.
+ * @param cumulate: Boolean flag to determine whether to cumulate the partial correlations.
+ * @return A vector of size 2 containing:
+ *         - rho[0]: Pearson correlation between the target and its predicted values.
+ *         - rho[1]: Partial correlation between the target and its predicted values, adjusting for control variables.
+ */
 std::vector<double> PartialSMap4Lattice(
-    const std::vector<std::vector<double>>& vectors,  // Reconstructed state-space (each row is a separate vector/state)
-    const std::vector<double>& target,                // Spatial cross-section series to be used as the target (should line up with vectors)
-    const std::vector<std::vector<double>>& controls, // Cross-sectional data of control variables (**stored by row**)
-    const std::vector<std::vector<int>>& nb_vec,      // Neighbor indices vector of the spatial units
-    const std::vector<bool>& lib_indices,             // Vector of T/F values (which states to include when searching for neighbors)
-    const std::vector<bool>& pred_indices,            // Vector of T/F values (which states to predict from)
-    const std::vector<int>& conEs,                    // Number of dimensions for the attractor reconstruction with control variables
-    int num_neighbors,                                // Number of neighbors to use for simplex projection
-    double theta,                                     // Weighting parameter for distances
-    bool cumulate,                                    // Whether to cumulate the partial correlations
-    bool includeself                                  // Whether to include the current state when constructing the embedding vector
+    const std::vector<std::vector<double>>& vectors,
+    const std::vector<double>& target,
+    const std::vector<std::vector<double>>& controls,
+    const std::vector<std::vector<int>>& nb_vec,
+    const std::vector<bool>& lib_indices,
+    const std::vector<bool>& pred_indices,
+    const std::vector<int>& conEs,
+    const std::vector<int>& taus,
+    int num_neighbors,
+    double theta,
+    bool cumulate
 ){
   int n_controls = controls.size();
   std::vector<double> rho(2);
@@ -93,7 +135,7 @@ std::vector<double> PartialSMap4Lattice(
       } else {
         temp_pred = SMapPrediction(temp_embedding, controls[i], lib_indices, pred_indices, num_neighbors, theta);
       }
-      temp_embedding = GenLatticeEmbeddings(temp_pred,nb_vec,conEs[i],includeself);
+      temp_embedding = GenLatticeEmbeddings(temp_pred,nb_vec,conEs[i],taus[i]);
     }
 
     std::vector<double> con_pred = SMapPrediction(temp_embedding, target, lib_indices, pred_indices, num_neighbors, theta);
@@ -108,7 +150,7 @@ std::vector<double> PartialSMap4Lattice(
 
     for (int i = 0; i < n_controls; ++i) {
       temp_pred = SMapPrediction(vectors, controls[i], lib_indices, pred_indices, num_neighbors, theta);
-      temp_embedding = GenLatticeEmbeddings(temp_pred,nb_vec,conEs[i],includeself);
+      temp_embedding = GenLatticeEmbeddings(temp_pred,nb_vec,conEs[i],taus[i]);
       temp_pred = SMapPrediction(temp_embedding, target, lib_indices, pred_indices, num_neighbors, theta);
       con_pred[i] = temp_pred;
     }
@@ -121,6 +163,32 @@ std::vector<double> PartialSMap4Lattice(
   return rho;
 }
 
+/*
+ * Perform SCPCM on a single library and prediction set for lattice data.
+ *
+ * Parameters:
+ *   - x_vectors: Reconstructed state-space (each row represents a separate vector/state).
+ *   - y: Spatial cross-section series used as the target (should align with x_vectors).
+ *   - controls: Cross-sectional data of control variables (stored by row).
+ *   - nb_vec: Neighbor indices vector of the spatial units.
+ *   - lib_indices: A boolean vector indicating which states to include when searching for neighbors.
+ *   - lib_size: Size of the library used for cross mapping.
+ *   - max_lib_size: Maximum size of the library.
+ *   - possible_lib_indices: Indices of possible library states.
+ *   - pred_indices: A boolean vector indicating which states to use for prediction.
+ *   - conEs: Number of dimensions for attractor reconstruction with control variables.
+ *   - taus: Spatial lag step for constructing lagged state-space vectors with control variables.
+ *   - b: Number of neighbors to use for simplex projection.
+ *   - simplex: If true, uses simplex projection for prediction; otherwise, uses s-mapping.
+ *   - theta: Distance weighting parameter for local neighbors in the manifold (used in s-mapping).
+ *   - cumulate: Whether to accumulate partial correlations.
+ *
+ * Returns:
+ *   A vector of PartialCorRes objects, where each contains:
+ *   - An integer representing the library size.
+ *   - A double representing the Pearson correlation coefficient (rho).
+ *   - A double representing the Partial correlation coefficient (pratial rho).
+ */
 std::vector<PartialCorRes> SCPCMSingle4Lattice(
     const std::vector<std::vector<double>>& x_vectors,  // Reconstructed state-space (each row is a separate vector/state)
     const std::vector<double>& y,                       // Spatial cross-section series to be used as the target (should line up with vectors)
@@ -132,12 +200,11 @@ std::vector<PartialCorRes> SCPCMSingle4Lattice(
     const std::vector<int>& possible_lib_indices,       // Indices of possible library states
     const std::vector<bool>& pred_indices,              // Vector of T/F values (which states to predict from)
     const std::vector<int>& conEs,                      // Number of dimensions for the attractor reconstruction with control variables
-    int E,                                              // Number of dimensions for the attractor reconstruction
+    const std::vector<int>& taus,                       // Spatial lag step for constructing lagged state-space vectors with control variables
     int b,                                              // Number of neighbors to use for simplex projection
     bool simplex,                                       // Algorithm used for prediction; Use simplex projection if true, and s-mapping if false
     double theta,                                       // Distance weighting parameter for the local neighbours in the manifold
-    bool cumulate,                                      // Whether to cumulate the partial correlations
-    bool includeself                                    // Whether to include the current state when constructing the embedding vector
+    bool cumulate                                       // Whether to cumulate the partial correlations
 ) {
   int n = x_vectors.size();
   std::vector<PartialCorRes> x_xmap_y;
@@ -151,9 +218,9 @@ std::vector<PartialCorRes> SCPCMSingle4Lattice(
 
     // Run partial cross map and store results
     if (simplex) {
-      rho = PartialSimplex4Lattice(x_vectors, y, controls, nb_vec, lib_indices, pred_indices, conEs, b, cumulate, includeself);
+      rho = PartialSimplex4Lattice(x_vectors, y, controls, nb_vec, lib_indices, pred_indices, conEs, taus, b, cumulate);
     } else {
-      rho = PartialSMap4Lattice(x_vectors, y, controls, nb_vec, lib_indices, pred_indices, conEs, b, theta, cumulate, includeself);
+      rho = PartialSMap4Lattice(x_vectors, y, controls, nb_vec, lib_indices, pred_indices, conEs, taus, b, theta, cumulate);
     }
     x_xmap_y.emplace_back(lib_size, rho[0], rho[1]);
   } else {
@@ -176,9 +243,9 @@ std::vector<PartialCorRes> SCPCMSingle4Lattice(
 
       // Run partial cross map and store results
       if (simplex) {
-        rho = PartialSimplex4Lattice(x_vectors, y, controls, nb_vec, lib_indices, pred_indices, conEs, b, cumulate, includeself);
+        rho = PartialSimplex4Lattice(x_vectors, y, controls, nb_vec, lib_indices, pred_indices, conEs, taus, b, cumulate);
       } else {
-        rho = PartialSMap4Lattice(x_vectors, y, controls, nb_vec, lib_indices, pred_indices, conEs, b, theta, cumulate, includeself);
+        rho = PartialSMap4Lattice(x_vectors, y, controls, nb_vec, lib_indices, pred_indices, conEs, taus, b, theta, cumulate);
       }
       x_xmap_y.emplace_back(lib_size, rho[0], rho[1]);
     }
@@ -187,28 +254,63 @@ std::vector<PartialCorRes> SCPCMSingle4Lattice(
   return x_xmap_y;
 }
 
+/**
+ * Performs SCPCM on a spatial lattice dataset.
+ *
+ * Parameters:
+ * - x: Spatial cross-section series used as the predictor variable (**cross mapping from**).
+ * - y: Spatial cross-section series used as the target variable (**cross mapping to**).
+ * - controls: Cross-sectional data of control variables (**stored by row**).
+ * - nb_vec: A nested vector containing neighborhood information for lattice data.
+ * - lib_sizes: A vector specifying different library sizes for SCPCM analysis.
+ * - lib: A vector specifying the library indices (1-based in R, converted to 0-based in C++).
+ * - pred: A vector specifying the prediction indices (1-based in R, converted to 0-based in C++).
+ * - Es: A vector specifying the embedding dimensions for attractor reconstruction using x and control variables.
+ * - taus: A vector specifying the spatial lag steps for constructing lagged state-space vectors using x and control variables.
+ * - b: Number of nearest neighbors used for prediction.
+ * - simplex: Boolean flag indicating whether to use simplex projection (true) or S-mapping (false) for prediction.
+ * - theta: Distance weighting parameter used for weighting neighbors in the S-mapping prediction.
+ * - threads: Number of threads to use for parallel computation.
+ * - cumulate: Boolean flag indicating whether to cumulate partial correlations.
+ * - progressbar: Boolean flag indicating whether to display a progress bar during computation.
+ *
+ * Returns:
+ *    A 2D vector of results, where each row contains:
+ *      - The library size.
+ *      - The mean pearson cross-mapping correlation.
+ *      - The statistical significance of the pearson correlation.
+ *      - The lower bound of the pearson correlation confidence interval.
+ *      - The upper bound of the pearson correlation confidence interval.
+ *      - The mean partial cross-mapping partial correlation.
+ *      - The statistical significance of the partial correlation.
+ *      - The lower bound of the partial correlation confidence interval.
+ *      - The upper bound of the partial correlation confidence interval.
+ */
 std::vector<std::vector<double>> SCPCM4Lattice(
     const std::vector<double>& x,                       // Spatial cross-section series to cross map from
     const std::vector<double>& y,                       // Spatial cross-section series to cross map to
     const std::vector<std::vector<double>>& controls,   // Cross-sectional data of control variables (**stored by row**)
     const std::vector<std::vector<int>>& nb_vec,        // Neighbor indices vector of the spatial units
     const std::vector<int>& lib_sizes,                  // Vector of library sizes to use
-    const std::vector<std::pair<int, int>>& lib,        // Matrix (n x 2) using n sequences of data to construct libraries
-    const std::vector<std::pair<int, int>>& pred,       // Matrix (n x 2) using n sequences of data to predict from
+    const std::vector<int>& lib,                        // Vector specifying the library indices
+    const std::vector<int>& pred,                       // Vector specifying the prediction indices
     const std::vector<int>& Es,                         // Number of dimensions for the attractor reconstruction with the x and control variables
-    int tau,                                            // Spatial lag for the lagged-vector construction
+    const std::vector<int>& taus,                       // Spatial lag step for constructing lagged state-space vectors with the x and control variables
     int b,                                              // Number of nearest neighbors to use for prediction
     bool simplex,                                       // Algorithm used for prediction; Use simplex projection if true, and s-mapping if false
     double theta,                                       // Distance weighting parameter for the local neighbours in the manifold
     int threads,                                        // Number of threads used from the global pool
     bool cumulate,                                      // Whether to cumulate the partial correlations
-    bool includeself,                                   // Whether to include the current state when constructing the embedding vector
-    bool progressbar = true                             // Whether to print the progress bar
+    bool progressbar                                    // Whether to print the progress bar
 ) {
   int Ex = Es[0];
   std::vector<int> conEs = Es;
   conEs.erase(conEs.begin());
 
+  int taux = taus[0];
+  std::vector<int> contaus = taus;
+  contaus.erase(contaus.begin());
+
   // If b is not provided correctly, default it to Ex + 2
   if (b <= 0) {
     b = Ex + 2;
@@ -218,7 +320,7 @@ std::vector<std::vector<double>> SCPCM4Lattice(
   unsigned int max_threads = std::thread::hardware_concurrency();
   threads_sizet = std::min(static_cast<size_t>(max_threads), threads_sizet);
 
-  std::vector<std::vector<double>> x_vectors = GenLatticeEmbeddings(x,nb_vec,Ex,includeself);
+  std::vector<std::vector<double>> x_vectors = GenLatticeEmbeddings(x,nb_vec,Ex,taux);
   int n = x_vectors.size();
 
   int n_confounds;
@@ -228,25 +330,52 @@ std::vector<std::vector<double>> SCPCM4Lattice(
     n_confounds = controls.size();
   }
 
-  // Setup pred_indices
+  // Initialize lib_indices and pred_indices with all false
+  std::vector<bool> lib_indices(n, false);
   std::vector<bool> pred_indices(n, false);
-  for (const auto& p : pred) {
-    int row_start = p.first + (Ex - 1) * tau;
-    int row_end = p.second;
-    if (row_end > row_start && row_start >= 0 && row_end < n) {
-      std::fill(pred_indices.begin() + row_start, pred_indices.begin() + row_end + 1, true);
-    }
+
+  // Convert lib and pred (1-based in R) to 0-based indices and set corresponding positions to true
+  int libsize_int = lib.size();
+  for (int i = 0; i < libsize_int; ++i) {
+    lib_indices[lib[i] - 1] = true; // Convert to 0-based index
+  }
+  int predsize_int = pred.size();
+  for (int i = 0; i < predsize_int; ++i) {
+    pred_indices[pred[i] - 1] = true; // Convert to 0-based index
   }
 
-  // Setup lib_indices
-  std::vector<bool> lib_indices(n, false);
-  for (const auto& l : lib) {
-    int row_start = l.first + (Ex - 1) * tau;
-    int row_end = l.second;
-    if (row_end > row_start && row_start >= 0 && row_end < n) {
-      std::fill(lib_indices.begin() + row_start, lib_indices.begin() + row_end + 1, true);
-    }
-  }
+  // /* Aligned with the previous implementation,
+  //  * now deprecated in the source C++ code.
+  //  *  ----- Wenbo Lv, written on 2025.02.10
+  //  */
+  // for (int i = 0, i < (Ex - 1) * tau, ++i){
+  //   lib_indices[lib[i] - 1] = false;
+  //   pred_indices[pred[i] - 1] = false;
+  // }
+
+  // /* Do not uncomment those codes;
+  //  * it's the previous implementation using `std::vector<std::pair<int, int>>`  input for lib and pred,
+  //  * kept for reference. ----- Wenbo Lv, written on 2025.02.09
+  //  */
+  // // Setup pred_indices
+  // std::vector<bool> pred_indices(n, false);
+  // for (const auto& p : pred) {
+  //   int row_start = p.first + (Ex - 1) * tau;
+  //   int row_end = p.second;
+  //   if (row_end > row_start && row_start >= 0 && row_end < n) {
+  //     std::fill(pred_indices.begin() + row_start, pred_indices.begin() + row_end + 1, true);
+  //   }
+  // }
+  //
+  // // Setup lib_indices
+  // std::vector<bool> lib_indices(n, false);
+  // for (const auto& l : lib) {
+  //   int row_start = l.first + (Ex - 1) * tau;
+  //   int row_end = l.second;
+  //   if (row_end > row_start && row_start >= 0 && row_end < n) {
+  //     std::fill(lib_indices.begin() + row_start, lib_indices.begin() + row_end + 1, true);
+  //   }
+  // }
 
   int max_lib_size = std::accumulate(lib_indices.begin(), lib_indices.end(), 0); // Maximum lib size
   std::vector<int> possible_lib_indices;
@@ -286,12 +415,11 @@ std::vector<std::vector<double>> SCPCM4Lattice(
   //     possible_lib_indices,
   //     pred_indices,
   //     conEs,
-  //     Ex,
+  //     contaus,
   //     b,
   //     simplex,
   //     theta,
-  //     cumulate,
-  //     includeself
+  //     cumulate
   //   );
   //   x_xmap_y.insert(x_xmap_y.end(), results.begin(), results.end());
   // }
@@ -312,12 +440,11 @@ std::vector<std::vector<double>> SCPCM4Lattice(
         possible_lib_indices,
         pred_indices,
         conEs,
-        Ex,
+        contaus,
         b,
         simplex,
         theta,
-        cumulate,
-        includeself
+        cumulate
       );
       x_xmap_y.insert(x_xmap_y.end(), results.begin(), results.end());
       bar++;
@@ -336,12 +463,11 @@ std::vector<std::vector<double>> SCPCM4Lattice(
         possible_lib_indices,
         pred_indices,
         conEs,
-        Ex,
+        contaus,
         b,
         simplex,
         theta,
-        cumulate,
-        includeself
+        cumulate
       );
       x_xmap_y.insert(x_xmap_y.end(), results.begin(), results.end());
     }, threads_sizet);
