@@ -6,14 +6,25 @@
 #include <limits>
 #include "CppStats.h"
 
-// Function to compute the 'simplex projection' prediction
-// Returns a vectors: target_pred
+/*
+ * Computes predictions using the simplex projection method based on state-space reconstruction.
+ *
+ * Parameters:
+ *   - vectors: Reconstructed state-space (each row represents a separate vector/state).
+ *   - target: Spatial cross-section series used as the target (should align with vectors).
+ *   - lib_indices: Vector of T/F values indicating which states to include when searching for neighbors.
+ *   - pred_indices: Vector of T/F values indicating which states to predict from.
+ *   - num_neighbors: Number of neighbors to use for simplex projection.
+ *
+ * Returns:
+ *   A vector<double> containing predicted target values (target_pred).
+ */
 std::vector<double> SimplexProjectionPrediction(
-    const std::vector<std::vector<double>>& vectors,  // Reconstructed state-space (each row is a separate vector/state)
-    const std::vector<double>& target,                // Spatial cross-section series to be used as the target (should line up with vectors)
-    const std::vector<bool>& lib_indices,             // Vector of T/F values (which states to include when searching for neighbors)
-    const std::vector<bool>& pred_indices,            // Vector of T/F values (which states to predict from)
-    int num_neighbors                                 // Number of neighbors to use for simplex projection
+    const std::vector<std::vector<double>>& vectors,
+    const std::vector<double>& target,
+    const std::vector<bool>& lib_indices,
+    const std::vector<bool>& pred_indices,
+    int num_neighbors
 ) {
   // Convert num_neighbors to size_t
   size_t num_neighbors_sizet = static_cast<size_t>(num_neighbors);
@@ -62,7 +73,10 @@ std::vector<double> SimplexProjectionPrediction(
       num_neighbors_sizet = neighbors.size();
     }
     std::partial_sort(neighbors.begin(), neighbors.begin() + num_neighbors_sizet, neighbors.end(),
-                      [&](size_t a, size_t b) { return distances[a] < distances[b]; });
+                      [&](size_t a, size_t b) {
+                        return (distances[a] < distances[b]) ||
+                               (distances[a] == distances[b] && a < b);
+                        });
 
     double min_distance = distances[neighbors[0]];
 
@@ -103,28 +117,46 @@ std::vector<double> SimplexProjectionPrediction(
   return pred;
 }
 
-// Rho value by the 'simplex projection' prediction
+/*
+ * Computes the Pearson correlation coefficient (rho) using the simplex projection prediction method.
+ *
+ * Parameters:
+ *   - vectors: Reconstructed state-space (each row represents a separate vector/state).
+ *   - target: Spatial cross-section series used as the target (should align with vectors).
+ *   - lib_indices: Vector of T/F values indicating which states to include when searching for neighbors.
+ *   - pred_indices: Vector of T/F values indicating which states to use for prediction.
+ *   - num_neighbors: Number of neighbors to use for simplex projection.
+ *
+ * Returns:
+ *   A double representing the Pearson correlation coefficient (rho) between the predicted and actual target values.
+ */
 double SimplexProjection(
-    const std::vector<std::vector<double>>& vectors,  // Reconstructed state-space (each row is a separate vector/state)
-    const std::vector<double>& target,                // Spatial cross-section series to be used as the target (should line up with vectors)
-    const std::vector<bool>& lib_indices,             // Vector of T/F values (which states to include when searching for neighbors)
-    const std::vector<bool>& pred_indices,            // Vector of T/F values (which states to predict from)
-    int num_neighbors                                 // Number of neighbors to use for simplex projection
+    const std::vector<std::vector<double>>& vectors,
+    const std::vector<double>& target,
+    const std::vector<bool>& lib_indices,
+    const std::vector<bool>& pred_indices,
+    int num_neighbors
 ) {
   std::vector<double> target_pred = SimplexProjectionPrediction(vectors, target, lib_indices, pred_indices, num_neighbors);
   return PearsonCor(target_pred, target, true);
 }
 
-// Description: Computes the simplex projection and returns a vector containing
-//              Pearson correlation coefficient (PearsonCor), mean absolute error (MAE),
-//              and root mean squared error (RMSE).
-// Parameters:
-//   - vectors: Reconstructed state-space (each row is a separate vector/state).
-//   - target: Spatial cross-section series to be used as the target (should align with vectors).
-//   - lib_indices: Vector of T/F values (which states to include when searching for neighbors).
-//   - pred_indices: Vector of T/F values (which states to predict from).
-//   - num_neighbors: Number of neighbors to use for simplex projection.
-// Returns: A vector<double> containing {PearsonCor, MAE, RMSE}.
+/*
+ * Computes the simplex projection and evaluates prediction performance.
+ *
+ * Parameters:
+ *   - vectors: Reconstructed state-space (each row is a separate vector/state).
+ *   - target: Spatial cross-section series to be used as the target (should align with vectors).
+ *   - lib_indices: Vector of T/F values (which states to include when searching for neighbors).
+ *   - pred_indices: Vector of T/F values (which states to predict from).
+ *   - num_neighbors: Number of neighbors to use for simplex projection.
+ *
+ * Returns:
+ *   A vector<double> containing:
+ *     - Pearson correlation coefficient (PearsonCor)
+ *     - Mean absolute error (MAE)
+ *     - Root mean squared error (RMSE)
+ */
 std::vector<double> SimplexBehavior(
     const std::vector<std::vector<double>>& vectors,
     const std::vector<double>& target,
